package fi.fa.opplugin;

import com.alibaba.druid.util.StringUtils;
import fi.fa.opplugin.utils.ChangeDataUtils;
import kd.bos.bill.BillShowParameter;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.form.ShowType;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.list.BillList;
import kd.bos.list.plugin.AbstractListPlugin;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import java.math.BigDecimal;
import java.util.EventObject;
import java.util.List;

//资产变更单中间表下推资产变更单
public class FaChangeMidOpPlugin extends AbstractListPlugin {

    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addItemClickListeners(new String[]{"toolbarap"});
    }

    @Override
    public void itemClick(ItemClickEvent evt) {
        String itemKey = evt.getItemKey();
        if ("nckd_push".equals(itemKey)) {
            StringBuilder errMsg = new StringBuilder();
            BillList billList = (BillList)this.getControl("billlistap");
            ListSelectedRowCollection list =  billList.getSelectedRows();
            Object[] primaryKeyValues = list.getPrimaryKeyValues();
            if (primaryKeyValues.length > 0) {
                for (Object pk : primaryKeyValues) {
                    DynamicObject info = BusinessDataServiceHelper.loadSingle(pk, "nckd_e_change_mid");
                    String nckd_flag = info.getString("nckd_flag");
                    String direction = info.getString("nckd_direction");
                    if ("1".equals(nckd_flag)) {
                        errMsg.append("单据号: ").append(info.getString("number")).append("已经下推生成了业务处理单，不允许重复下推!");
                        errMsg.append("\r\n");
                        continue;
                    }
                    String error = ChangeDataUtils.check(info);
                    if (!StringUtils.isEmpty(error)) {
                        errMsg.append(error);
                        errMsg.append("\r\n");
                        continue;
                    }
                    DynamicObject busProcess = BusinessDataServiceHelper.newDynamicObject("nckd_ywcld");
                    busProcess.set("nckd_bizdate", info.getDate("nckd_date_out"));//业务日期

                    Long orgIdin = ChangeDataUtils.getOrg(info.getString("nckd_org_in"));
                    Long relationIdin = ChangeDataUtils.getRelation(orgIdin);
                    busProcess.set("nckd_costcompany", relationIdin);//资产组织
                    busProcess.set("nckd_orgfield", relationIdin);//核算组织
                    busProcess.set("nckd_textfield1", info.getString("nckd_reason"));//调拨原因
                    busProcess.set("billstatus", "A");//暂存
                    busProcess.set("nckd_detailtype", "N");//业务类型(E:资产调拨)
                    busProcess.set("nckd_mid_assert", pk);//中间表id
                    busProcess.set("nckd_source_type", "nckd_e_change_mid");//源单类型
                    busProcess.set("creator", ChangeDataUtils.getUser(info.getString("nckd_user_out")));//创建人
                    busProcess.set("nckd_applierv", ChangeDataUtils.getUser(info.getString("nckd_user_out")));//调拨申请人

                    List<String> codeList =ChangeDataUtils.getCardNumber(info);
                    QFilter filter = new QFilter("number", QCP.in,codeList);
                    filter.and("isbak",  QCP.equals, "0");
                    DynamicObject[] card = BusinessDataServiceHelper.load("fa_card_real","id",new QFilter[] {filter});
                    for (int c = 0; c < card.length; c++) {
                        DynamicObject cardInfo = BusinessDataServiceHelper.loadSingle(card[c].getPkValue(), card[c].getDynamicObjectType().getName());
                        String assetname = cardInfo.getString("assetname");//资产名称
                        DynamicObject headusedept = cardInfo.getDynamicObject("headusedept");//变更前部门
                        DynamicObject headuseperson = cardInfo.getDynamicObject("headuseperson");//变更前使用人
                        String nckd_storeplace = cardInfo.getString("nckd_storeplace");//变更前存放地点
                        DynamicObject supplier = cardInfo.getDynamicObject("supplier");//供应商
                        DynamicObject assetcat = cardInfo.getDynamicObject("assetcat");//资产类别

                        //创建资产调拨分录
                        DynamicObjectCollection entryEntity = busProcess.getDynamicObjectCollection("nckd_entryentity681");
                        DynamicObject entry1 = entryEntity.addNew();
                        entry1.set("nckd_basedatafield21", cardInfo);//资产编码
                        entry1.set("nckd_textfield43", assetname);
                        entry1.set("nckd_textfield44", assetname);
                        entry1.set("nckd_userfield", headuseperson);//调入使用人
                        entry1.set("nckd_userfield1", ChangeDataUtils.getUser(info.getString("nckd_used_user")));//调入使用人
                        entry1.set("nckd_textfield42", nckd_storeplace);
                        entry1.set("nckd_textfield45", info.getString("nckd_storeplace"));//调入存放地点
                        entry1.set("nckd_orgfield7", headusedept);//变更后使用部门
                        entry1.set("nckd_orgfield8", ChangeDataUtils.getOrg(info.getString("nckd_dept_in")));//变更后使用部门
                        entry1.set("nckd_basedatafield24", supplier);
                        entry1.set("nckd_basedatafield25", supplier);
                        entry1.set("nckd_basedatafield26", assetcat);
                        entry1.set("nckd_basedatafield27", assetcat);
                    }

                    //执行保存-提交-审核
                    OperationResult saveResult = SaveServiceHelper.saveOperate("nckd_ywcld", new DynamicObject[]{busProcess}, OperateOption.create());
                    if (saveResult.isSuccess()){
                        info.set("nckd_flag", "1");
                        info.set("nckd_ywcl", saveResult.getSuccessPkIds().get(0));
                        SaveServiceHelper.save(new DynamicObject[]{info});
                        errMsg.append("单据号: ").append(info.getString("number")).append("下推生成业务处理单成功!");
                        errMsg.append("\r\n");
                        OperationResult submitResult = OperationServiceHelper.executeOperate("submit", "nckd_ywcld",saveResult.getSuccessPkIds().toArray(),OperateOption.create());
                        OperationResult auditResult = OperationServiceHelper.executeOperate("audit", "nckd_ywcld",submitResult.getSuccessPkIds().toArray(),OperateOption.create());
                    }else {
                        errMsg.append(saveResult.getMessage());
                        errMsg.append("\r\n");
                    }
                }
            }else {
                this.getView().showMessage("请至少选择一条数据");
            }
            this.getView().showMessage(errMsg.toString());
        } else if ("nckd_trackdown".equals(itemKey)) {
            BillList billList = (BillList) this.getControl("billlistap");
            ListSelectedRowCollection list = billList.getSelectedRows();
            if (list.size() == 0) {
                this.getView().showMessage("请选择一条数据");
                return;
            } else if (list.size() > 1) {
                this.getView().showMessage("只能选择一条数据");
                return;
            }
            Object[] primaryKeyValues = list.getPrimaryKeyValues();
            long id = 0;
            for (Object pk : primaryKeyValues) {
                DynamicObject info = BusinessDataServiceHelper.loadSingle(pk, "nckd_e_change_mid");
                id = info.getLong("nckd_ywcl");
                if (id == 0) {
                    this.getView().showMessage("该单据还未下推生成业务处理单");
                    return;
                }
            }
            BillShowParameter showParameter = new BillShowParameter();

            showParameter.setFormId("nckd_ywcld");

            showParameter.getOpenStyle().setShowType(ShowType.Modal);//打开方式

            showParameter.setPkId(id);

            this.getView().showForm(showParameter);
        }
    }
}
